import sys
import nanolog as nl
import pytest


def test_time2str():
    print(nl.time2str('current time: YMD HMS'))
    print(nl.time2str('MY@HM'))
    print(nl.time2str('YMD, MD, YM, HM, MS'))


@pytest.mark.parametrize('backend', ['builtin', 'thirdparty'])
def test_prettyprint(backend):
    nl.set_pprint_backend(backend)
    l1 = ['spam', 'eggs', 'lumberjack', 'knights', 'ni']
    l1.insert(0, l1[:])
    l2 = ['SPAM', 'EGGS', 'LUMBERJACK', 'KNIGHTS', 'NI']
    l2.insert(0, l2)

    nl.pprint(l1, '<->', l2, '<=>', l1, end='\n--\n', width=30, compact=True)

    nl.pprintfmt('{0}<->{var:.3f}<=>{1}<->{0}',
                 l1, l2, var=1/7.,
                 end='\n--\n', width=30, compact=True)


def test_pbanner():
    nl.pbanner('my', 3, 'world', symbol='!', banner_len=16, banner_lines=3)
    nl.pbanner(banner_len=16, banner_lines=2)
    nl.pbanner('my', 'critical', 'case', symbol='<*_*>',
               banner_len=16, banner_lines=6, file=sys.stderr)
    nl.pbannerfmt('{} {:0>5d} {var}',
                  'hello', 333, var=777,
                  symbol='!', banner_len=16, banner_lines=3)
    nl.pbannerfmt('{3}&{0}&{2}&{1}',
                  'a', 'b', 'c', 'd',
                  symbol='%#@.@', banner_len=16, banner_lines=6, file=sys.stderr)


def test_compact():
    data = {'10.22.197.3': [{'DBClientID': '2996e1a93ee6a9234808fa04e5a889ae2e1984f0', 'ClientType': 'local_scheduler', 'Deleted': False, 'LocalSchedulerSocketName': '/tmp/scheduler56853620', 'GPU': 0.0, 'CPU': 8.0, 'mujoco': 15.0, 'AuxAddress': '10.22.197.3:53895'}, {'DBClientID': '860f9b328bde40a5bae565943f76fc0ce8bcd640', 'ClientType': 'plasma_manager', 'Deleted': False, 'store_socket_name': '/tmp/plasma_store63002382', 'manager_socket_name': '/tmp/plasma_manager2032735', 'AuxAddress': '10.22.197.3:53895'}], '10.22.193.6': [{'DBClientID': '15d9e88a0228d07934f31ce5cfe3835afb94497d', 'ClientType': 'global_scheduler', 'Deleted': False}, {'DBClientID': '63fba70f2d7fd35611329b93f7c6f1c888c69cc4', 'ClientType': 'local_scheduler', 'Deleted': False, 'LocalSchedulerSocketName': '/tmp/scheduler73889199', 'GPU': 0.0, 'CPU': 2.0, 'AuxAddress': '10.22.193.6:25163'}, {'DBClientID': '90e66ecf88a7cfb5972bbdc31776aacbb022578b', 'ClientType': 'plasma_manager', 'Deleted': False, 'store_socket_name': '/tmp/plasma_store84158689', 'manager_socket_name': '/tmp/plasma_manager95601982', 'AuxAddress': '10.22.193.6:25163'}], '10.22.195.3': [{'DBClientID': 'df540889a36dc1324d6a85bf123f1f7f4f6ea1ba', 'ClientType': 'local_scheduler', 'Deleted': False, 'LocalSchedulerSocketName': '/tmp/scheduler86198036', 'GPU': 0.0, 'CPU': 8.0, 'mujoco': 15.0, 'AuxAddress': '10.22.195.3:57600'}, {'DBClientID': '9f569963466abda68b7d90192789e3fda2f32f8f', 'ClientType': 'plasma_manager', 'Deleted': False, 'store_socket_name': '/tmp/plasma_store77322023', 'manager_socket_name': '/tmp/plasma_manager90407822', 'AuxAddress': '10.22.195.3:57600'}], '10.22.196.3': [{'DBClientID': 'fb700431d91769c5c63d5f380a71085fac914b8f', 'ClientType': 'local_scheduler', 'Deleted': False, 'LocalSchedulerSocketName': '/tmp/scheduler78187679', 'GPU': 0.0, 'CPU': 8.0, 'mujoco': 15.0, 'AuxAddress': '10.22.196.3:49323'}, {'DBClientID': '13da60a13c5bf2238296df0d953938c6c39073b6', 'ClientType': 'plasma_manager', 'Deleted': False, 'store_socket_name': '/tmp/plasma_store78738634', 'manager_socket_name': '/tmp/plasma_manager24412265', 'AuxAddress': '10.22.196.3:49323'}]}

    # if compact is True, always delegate to builtin pprint module
    nl.pprint(data, width=300, compact=True)
    nl.pprint(data, width=300, indent=4)


def test_ppfmt():
    d1 = {'a': {'a': {'a': {'a': {'a': {'b': 10}}}}}}
    d2 = {'A': {'A': {'A': {'A': {'A': {'B': 10}}}}}}

    nl.ppf('D2->{2}, num={1:.3f}, D1->{0}',
           d1, 1/7, d2,
           width=10, depth=4, compact=False)
    nl.ppf('compact D2->{2}, num={1:.3f}, D1->{0}',
           d1, 1/7, d2,
           width=10, depth=3, compact=True)
    nl.ppf('{myd2} myerr {myd2}', myd2=d2, width=35)



"""
Printing utils:

- Context manager-based printing streams.
- Convert data structures to pretty string representation. 
"""

import os
import sys
import datetime
import time
import pprint as _pprint_builtin
import prettyprinter as _pprint_thirdparty
import numbers
import inspect
from collections import abc
from io import StringIO
import traceback
from .constants import PP_DEFAULT


_PP_BACKEND = _pprint_thirdparty
# global default configs
_PP_CONFIG = {
    'indent': 1,
    'width': 80,
    'depth': None,
    'compact': False
}


def set_pprint_backend(backend):
    """
    Args:
        backend (string):
        - "builtin" for python builtin prettyprint
        - "thirdparty" (default) for Kaikkonen's prettyprinter
            https://github.com/tommikaikkonen/prettyprinter
    """
    global _PP_BACKEND
    backend = backend.lower()
    assert backend in ['builtin', 'thirdparty']
    if backend == 'builtin':
        _PP_BACKEND = _pprint_builtin
    else:
        _PP_BACKEND = _pprint_thirdparty


def set_pprint_config(indent=PP_DEFAULT,
                      width=PP_DEFAULT,
                      depth=PP_DEFAULT,
                      compact=PP_DEFAULT):
    kwargs = dict(indent=indent, width=width, depth=depth, compact=compact)
    for key, value in kwargs.items():
        if value != PP_DEFAULT:
            _PP_CONFIG[key] = value


def _pp_obj_str(obj, indent, width, depth, compact, *, _leave_number):
    """
    pprint a single obj, helper to pprint and pprintfmt
    If compact is True, always use the builtin backend cause thirdparty doesn't support
    """
    if isinstance(obj, str):
        return obj
    # don't convert number to string if we pass it to str.format()
    if isinstance(obj, numbers.Number) and _leave_number:
        return obj
    kwargs = dict(indent=indent, width=width, depth=depth, compact=compact)
    for key, value in kwargs.items():
        if value == PP_DEFAULT:
            kwargs[key] = _PP_CONFIG[key]
    backend = _PP_BACKEND
    if kwargs['compact']:
        backend = _pprint_builtin
    return backend.pformat(obj, **kwargs)


def printerr(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def printfmt(msg, *fmt_args,
             end='\n', file=sys.stdout, flush=False,
             **fmt_kwargs):
    msg = msg.format(*fmt_args, **fmt_kwargs)
    print(msg, end=end, file=file, flush=flush)


def printstr(*args, **kwargs):
    """
    Print to a string.
    Does not include the trailing newline unless you specify end='\n' explicitly
    """
    buf = StringIO()
    if 'end' not in kwargs:
        kwargs['end'] = ''
    print(*args, file=buf, **kwargs)
    return buf.getvalue()


def printfmterr(msg, *fmt_args,
                end='\n', flush=False,
                **fmt_kwargs):
    printfmt(msg, *fmt_args,
             end=end, file=sys.stderr, flush=flush,
             **fmt_kwargs)


def pprint(*objs,
           sep=' ', end='\n', file=sys.stdout, flush=False,
           indent=PP_DEFAULT, width=PP_DEFAULT, depth=PP_DEFAULT, compact=PP_DEFAULT
           ):
    """
    print() with prettyprint
    """
    print(
        pprintstr(*objs, sep=sep,
                  indent=indent, width=width, depth=depth, compact=compact),
        end=end, file=file, flush=flush
    )


def pprintstr(*objs, sep=' ',
              indent=PP_DEFAULT, width=PP_DEFAULT, depth=PP_DEFAULT, compact=PP_DEFAULT
              ):
    pf = lambda obj: _pp_obj_str(
        obj, indent, width, depth, compact, _leave_number=False)
    return sep.join(map(pf, objs))


def pprintfmt(msg, *fmt_args,
              end='\n', file=sys.stdout, flush=False,
              indent=PP_DEFAULT, width=PP_DEFAULT, depth=PP_DEFAULT, compact=PP_DEFAULT,
              **fmt_kwargs):
    """
    Equivalent to print(msg.format(*args, **kwargs)) with prettyprint
    """
    print(
        pprintfmtstr(msg, *fmt_args,
                     indent=indent, width=width, depth=depth, compact=compact,
                     **fmt_kwargs),
        end=end, file=file, flush=flush
    )


def pprintfmtstr(msg, *fmt_args,
                 indent=PP_DEFAULT, width=PP_DEFAULT, depth=PP_DEFAULT, compact=PP_DEFAULT,
                 **fmt_kwargs):
    """
    Equivalent to print(msg.format(*args, **kwargs)) with prettyprint

    Warnings:
        all positional and keyword args to str.format will be converted to string
        first, which means float/int will not be
    """
    pf = lambda obj: _pp_obj_str(
        obj, indent, width, depth, compact, _leave_number=True)
    fmt_args = list(map(pf, fmt_args))
    fmt_kwargs = {key: pf(value) for key, value in fmt_kwargs.items()}
    return msg.format(*fmt_args, **fmt_kwargs)


# ---------------- shorthands -----------------
pstr = printstr
perr = printerr
pf = printfmt
pferr = printfmterr
pp = pprint
ppstr = pprintstr
ppf = pprintfmt
ppfstr = pprintfmtstr


# ---------------- banners -----------------
def _banner(msg, symbol, banner_len, banner_lines):
    "helper for banner() and bannerfmt()"
    # repeat `symbol`, cut in middle if necessary
    half_len = banner_len // 2
    half_banner = (symbol * (half_len // len(symbol))
                   + symbol[:half_len % len(symbol)])
    assert len(half_banner) == half_len, 'INTERNAL ERROR'
    central_line = u'{half_banner}{space}{msg}{space}{half_banner}'.format(
        half_banner=half_banner,
        msg=msg,
        space=' ' if msg else ''
    )
    full_len = len(central_line)  # including message
    banner_lines -= 1
    surround_banner = (symbol * (full_len // len(symbol))
                       + symbol[:full_len % len(symbol)])
    before_banner = [surround_banner] * (banner_lines // 2)
    after_banner = [surround_banner] * (banner_lines - banner_lines // 2)
    full_banner = before_banner + [central_line] + after_banner
    return '\n'.join(full_banner)


def banner(*msg, sep=' ', symbol='=', banner_len=20, banner_lines=1):
    """
    A banner line or block with your message in the middle

    Args:
      sep: separator between *msg, same as in print()
      symbol: banner symbol
      banner_len: length of the banner symbols (excluding message itself)
      banner_lines: number of the banner lines, ideally an odd number

    Example:
      # long unbroken line of '!'
      banner(symbol='!', bannerlen=80)
      # !!!!!!!!! my hello world !!!!!!!!!
      banner('my', 'hello', 'world', symbol='!', banner_len=10)
    """
    msg = sep.join([u'{}'] * len(msg)).format(*msg)
    return _banner(msg, symbol, banner_len, banner_lines)


def pbanner(*msg, sep=' ', symbol='=', banner_len=20, banner_lines=1,
            end='\n', file=sys.stdout, flush=False):
    print(banner(*msg, sep=sep,
        symbol=symbol, banner_len=banner_len, banner_lines=banner_lines
    ), end=end, file=file, flush=flush)


def bannerfmt(msg, *fmt_args, symbol='=', banner_len=20, banner_lines=1,
              **fmt_kwargs):
    """
    A banner line or block with your message in the middle.
    Message is formatted in {}-style with *args and **kwargs
    Other banner settings are the same as banner() method

    Args:
      msg: "{}"-style format string
      *fmt_args: positional args for the format string
      **fmt_kwargs: keyword args for the format string
      symbol: banner symbol
      banner_len: length of the banner symbols (excluding message itself)
      banner_lines: number of the banner lines, ideally an odd number
    """
    msg = msg.format(*fmt_args, **fmt_kwargs)
    return _banner(msg, symbol, banner_len, banner_lines)


def pbannerfmt(msg, *fmt_args, symbol='=', banner_len=20, banner_lines=1,
               end='\n', file=sys.stdout, flush=False, **fmt_kwargs):
    print(bannerfmt(msg, *fmt_args,
        symbol=symbol, banner_len=banner_len, banner_lines=banner_lines, **fmt_kwargs
    ), end=end, file=file, flush=flush)


# ---------------- time formatting -----------------
def get_time_formatter(formatter):
    """
    Convenient aliases for common time formatter.
    For complete time format spec, please refer to
    https://docs.python.org/3/library/time.html#time.strftime

    Args:
      formatter: y=year, d=day, h=hour, m=minute, s=second
        any combination of
        'MDY' 'YMD' 'DMY' 'YDM' 'MY' 'YM' 'MD' 'DM'
        'HMS' 'HM' 'MS'
        formatted as in "month-day-year" and "hour:minute:second"

    Examples:
      "MDY HMS" => "12-25-18 16:38:05"
      "HM@MD" => "16:38@12-25"
      "time: YMD MS" => "time: 18-12-25 38:05"

    Returns:
      time format string with %
    """
    if formatter is None:
        return None
    return (
        formatter
            .replace('MDY', '%m-%d-%y')
            .replace('YMD', '%y-%m-%d')
            .replace('DMY', '%d-%m-%y')
            .replace('YDM', '%y-%d-%m')
            .replace('MY', '%m-%y')
            .replace('YM', '%y-%m')
            .replace('MD', '%m-%d')
            .replace('DM', '%d-%m')
            .replace('HMS', '%H:%M:%S')
            .replace('HM', '%H:%M')
            .replace('MS', '%M:%S')
    )


def time2str(formatter):
    """
    https://docs.python.org/3/library/time.html#time.strftime
    %m - month; %d - day; %y - year
    %H - 24 hr; %I - 12 hr; %M - minute; %S - second; %p - AM or PM

    See `get_time_formatter` for convenient aliases in formatter

    Returns:
        string of the current time formatted with `formatter`
    """
    if '%' not in formatter:
        formatter = get_time_formatter(formatter)
    return time.strftime(formatter)


def seconds2str(seconds):
    "Convert seconds to str `HH:MM:SS`"
    return datetime.timedelta(seconds=seconds)


# ---------------- data struct to str -----------------
def dict2str(D,
             sep='=',
             item_sep=', ',
             key_format='',
             value_format='',
             enclose=('{', '}')):
    """
    Pretty string representation of a dictionary. Works with Unicode.

    Args:
      sep: "key `sep` value"
      item_sep: separator between key-value pairs
      key_format: same format string as in str.format()
      value_format: same format string as in str.format()
      enclose: a 2-tuple of enclosing symbols
    """
    assert len(enclose) == 2
    itemstrs = []
    for key, value in D.items():
        itemstrs.append(u'{{:{}}} {} {{:{}}}'
                        .format(key_format, sep, value_format)
                        .format(key, value))
    return enclose[0] + item_sep.join(itemstrs) + enclose[1]


def list2str(L,
             sep=', ',
             item_format='',
             enclose=None):
    """
    Pretty string representation of a list or tuple. Works with Unicode.

    Args:
      sep: separator between two list items
      item_format: same format string as in str.format()
      enclose: a 2-tuple of enclosing symbols. 
          default: `[]` for list and `()` for tuple.
    """
    if enclose is None:
        if isinstance(L, tuple):
            enclose = ('(', ')')
        else:
            enclose = ('[', ']')
    else:
        assert len(enclose) == 2
    item_format = u'{{:{}}}'.format(item_format)
    itemstr = sep.join(map(lambda s: item_format.format(s), L))
    return enclose[0] + itemstr + enclose[1]


def exception2str(exc):
    """
    Returns
        string of the traceback message
    """
    buf = StringIO()
    traceback.print_exception(
        type(exc),
        exc,
        exc.__traceback__,
        file=buf
    )
    return buf.getvalue().strip()


def signature2str(func):
    return str(inspect.signature(func))


# ---------------- print redirections -----------------
class PrintRedirection:
    """
    Context manager: temporarily redirects stdout and stderr
    """
    def __init__(self, stdout=None, stderr=None):
        """
        Args:
          stdout: if None, defaults to sys.stdout, unchanged
          stderr: if None, defaults to sys.stderr, unchanged
        """
        if stdout is None:
            stdout = sys.stdout
        if stderr is None:
            stderr = sys.stderr
        self._stdout, self._stderr = stdout, stderr

    def __enter__(self):
        self._old_out, self._old_err = sys.stdout, sys.stderr
        self._old_out.flush()
        self._old_err.flush()
        sys.stdout, sys.stderr = self._stdout, self._stderr
        return self
            
    def __exit__(self, exc_type, exc_value, traceback):
        self.flush()
        # restore the normal stdout and stderr
        sys.stdout, sys.stderr = self._old_out, self._old_err
    
    def flush(self):
        "Manually flush the replaced stdout/stderr buffers."
        self._stdout.flush()
        self._stderr.flush()


class PrintFile(PrintRedirection):
    """
    Print to file and save/close the handle at the end.
    """
    def __init__(self, out_file=None, err_file=None):
        """
        Args:
          out_file: file path
          err_file: file path. If the same as out_file, print both stdout 
              and stderr to one file in order.
        """
        self.out_file, self.err_file = out_file, err_file
        if out_file:
            out_file = os.path.expanduser(out_file)
            self.out_file = open(out_file, 'w')
        if err_file:
            err_file = os.path.expanduser(err_file)
            if err_file == out_file: # redirect both stdout/err to one file
                self.err_file = self.out_file
            else:
                self.err_file = open(os.path.expanduser(err_file), 'w')

        super().__init__(stdout=self.out_file, stderr=self.err_file)
    
    def __exit__(self, *args):
        super().__exit__(*args)
        if self.out_file:
            self.out_file.close()
        if self.err_file:
            self.err_file.close()


def PrintSuppress(no_out=True, no_err=True):
    """
    Args:
      no_out: stdout writes to sys.devnull
      no_err: stderr writes to sys.devnull
    """
    out_file = os.devnull if no_out else None
    err_file = os.devnull if no_err else None
    return PrintFile(out_file=out_file, err_file=err_file)


class PrintString(PrintRedirection):
    """
    Redirect stdout and stderr to strings.
    """
    def __init__(self):
        self.out_stream = StringIO()
        self.err_stream = StringIO()
        super().__init__(stdout=self.out_stream, stderr=self.err_stream)
    
    def stdout(self):
        "Returns: stdout as one string."
        return self.out_stream.getvalue()
    
    def stderr(self):
        "Returns: stderr as one string."
        return self.err_stream.getvalue()
        
    def stdout_by_line(self):
        "Returns: a list of stdout line by line, ignore trailing blanks"
        return self.stdout().rstrip().split('\n')

    def stderr_by_line(self):
        "Returns: a list of stderr line by line, ignore trailing blanks"
        return self.stderr().rstrip().split('\n')




PP_DEFAULT = '_default_'  # use default prettyprint configs

LOG_ALL = 0
TRACE = 5
DEBUG = 10
DEBUG1 = 11
DEBUG2 = 12
DEBUG3 = 13
DEBUG4 = 14
DEBUG5 = 15
DEBUG6 = 16
DEBUG7 = 17
DEBUG8 = 18
DEBUG9 = 19
INFO = 20
INFO1 = 21
INFO2 = 22
INFO3 = 23
INFO4 = 24
NOTICE = 25
INFO5 = 25
INFO6 = 26
INFO7 = 27
INFO8 = 28
INFO9 = 29
WARNING = 30
WARN = 30
WARN1 = 31
WARN2 = 32
WARN3 = 33
WARN4 = 34
WARN5 = 35
WARN6 = 36
WARN7 = 37
WARN8 = 38
WARN9 = 39
ERROR = 40
ERROR1 = 41
ERROR2 = 42
ERROR3 = 43
ERROR4 = 44
ERROR5 = 45
ERROR6 = 46
ERROR7 = 47
ERROR8 = 48
ERROR9 = 49
CRITICAL = 50
CRITICAL1 = 51
CRITICAL2 = 52
CRITICAL3 = 53
CRITICAL4 = 54
CRITICAL5 = 55
CRITICAL6 = 56
CRITICAL7 = 57
CRITICAL8 = 58
CRITICAL9 = 59
LOG_OFF = 100


if __name__ == '__main__':
    from nanolog.logger import _NAME2LEVEL
    # generate this file
    for name, value in sorted(_NAME2LEVEL.items(), key=lambda x: x[1]):
        print('{} = {}'.format(name, value))




import pytest
import logging
import inspect
import nanolog as nl


def test_level_name():
    assert logging.getLevelName(20) == 'INFO'
    assert logging.getLevelName(23) == 'INFO3'
    assert logging.getLevelName(15) == 'DEBUG5'
    assert logging.getLevelName(37) == 'WARN7'
    assert logging.getLevelName(61) == 'LEVEL61'
    assert nl.get_level_number('INFO') == 20
    assert nl.get_level_number('INFO3') == 23
    assert nl.get_level_number('WARN7') == 37
    assert nl.get_level_number('DEBUG5') == 15
    assert nl.get_level_number('LEVEL17') == 17


@pytest.fixture
def logger():
    return nl.Logger.create_logger(
        'main',
        stream='out',
        level='debug5',
        show_level=False,
        format='{name} {asctime} {filename:>16s} {funcName}() {lineno} ${levelname}$\n',
        time_format='MDY@HMS',
    )


def test_doc(logger):
    print(inspect.getdoc(logger.debug))
    print(inspect.getdoc(logger.warnfmt5))
    print(inspect.getdoc(logger.infobanner))
    print(inspect.getdoc(logger.errorbanner2))
    print(inspect.getdoc(logger.criticalpp))
    print(inspect.getdoc(logger.debugppfmt3))


def test_log(logger):
    # print([_method for _method in dir(logger) if not _method.startswith('__')])
    logger.info7('my', 3, 'world', 1/16.)  # just like print
    logger.warn9('yo {:.3f} info7', 1/17)  # should NOT format!
    logger.debug3('SHOULD NOT SHOW!! logger level >= debug5')

    logger.error('this', 'an', 'error')
    try:
        1/0
    except Exception as e:
        logger.exception('myexc', 'yo', exc=e)
    logger.banner(logger.WARN4, 'yoyoyoo', symbol='%')
    logger.critical('yo crit {:.2e} {}', 3**0.5, {'x':3})


def test_logfmt(logger):
    logger.noticefmt('yo {:.3f} info7', 1/17)
    logger.criticalfmt2('yo crit {:.2e} {}', 3**0.5, {'x':3})
    logger.warningfmt('{}, we are {:.3f} miles from {planet}',
                      'Houston', 17/7, planet='Mars')  # just like str.format
    logger.errorfmt('Client format {:0>5d} - {:?<9}', 21, 'asdiojfoigj')
    logger.debugfmt5('debugger {:0>5d} - {:?<9}', 42, 'asodjfaisdfisaj')


def test_banner(logger):
    logger.noticebanner('my', 3, 'world', symbol='!',
                        banner_len=16, banner_lines=3)
    logger.infobanner3(banner_len=16, banner_lines=2)
    logger.criticalbanner('my', 'critical', 'case', symbol='<*_*>',
                          banner_len=16, banner_lines=6)
    logger.debugbanner('SHOULD NOT SHOW!! logger level >= debug5')


def test_bannerfmt(logger):
    logger.infobannerfmt3('{} {:0>5d} {var}',
                          'hello', 333, var=777,
                          symbol='!', banner_len=16, banner_lines=3)
    logger.errorbannerfmt('{3}&{0}&{2}&{1}',
                          'a', 'b', 'c', 'd',
                          symbol='<*_*>', banner_len=16, banner_lines=6)


def test_pp(logger):
    d1 = {'a': {'a': {'a': {'a': {'a': {'b': 10}}}}}}
    d2 = {'A': {'A': {'A': {'A': {'A': {'B': 10}}}}}}
    logger.debugpp5(d2, '<->', d1, compact=True)
    logger.warningpp('hello', d1, '<->', d2, width=10, depth=3)
    logger.errorpp7('<-err->', d2, width=35, compact=True)


def test_ppfmt(logger):
    d1 = {'a': {'a': {'a': {'a': {'a': {'b': 10}}}}}}
    d2 = {'A': {'A': {'A': {'A': {'A': {'B': 10}}}}}}
    logger.debugppfmt2('SHOULD', 'NOT', 'SHOW', d1)
    logger.warnppfmt5('compact D2->{2}, num={1:.3f}, D1->{0}',
                         d1, 1/7, d2,
                         width=10, depth=3, compact=True)
    logger.warnppfmt5('D2->{2}, num={1:.3f}, D1->{0}',
                         d1, 1/7, d2,
                         width=10, depth=3)
    logger.criticalppfmt('{myd2} myerr {myd2}', myd2=d2,
                         width=35, compact=True)


def test_set_level(logger):
    with logger.temp_level_scope(logger.ERROR5):
        logger.error4('SHOULD', 'NOT', 'DISPLAY')
    logger.error4('my', 20, 'hello', 30, 'world')
    with logger.temp_level_scope(logger.LOG_ALL):
        logger.trace('finest', 'grain', 'msgs')
    logger.trace('SHOULD', 'NOT', 'DISPLAY')
    with logger.temp_level_scope(logger.LOG_OFF):
        logger.critical9('LOG', 'TURNED', 'OFF')


